'+--------------------------------------------------------------------------+
'|                                Boing2                                    |
'|                                                                          |
'|             Example of 32-bit OpenGL layered splash screen               |
'|                                                                          |
'+--------------------------------------------------------------------------+
'|                                                                          |
'|                         Author Patrice TERRIER                           |
'|                            copyright(c) 2008                             |
'|                           www.zapsolution.com                            |
'|                        pterrier@zapsolution.com                          |
'|                                                                          |
'+--------------------------------------------------------------------------+
'|                  Project started on : 10-10-2008 (MM-DD-YYYY)            |
'|                        Last revised : 12-16-2014 (MM-DD-YYYY)            |
'+--------------------------------------------------------------------------+

#COMPILE EXE "Boing2.exe"
#INCLUDE "boing2.inc"
#RESOURCE "boing2.pbr"

declare function zTrace lib "zTrace.DLL" alias "zTrace" (zMessage as asciiz) as long

'------------------------------------------------------------------------------------------
' local section
'-----------------------------------------------------------------------------------------
%FRAME_SizeX = 200 '// The animation frame width
%FRAME_SizeY = 200 '// The animation frame height
%TIMER_DELAY = 20  '// The timer delay (animation speed)
%MOVE_STEP   = 8   '// The gP.x, gP.y step to move the window

type PROP
    x      as long
    y      as long
    sphere as long
    gldc   as dword
    glrc   as dword
    img    as dword
    mt     as BBPTEXTURE
end type

global gP as PROP

sub DrawAndSetupAlphaChannel(byval hMemDC as long, byval x as long, byval y as long)
    '// We must do this to be compatible with BitBlt
    '// We compute a fake alpha channel based on color conversion to shade of gray
    local pBits as byte PTR, bm as BITMAP, P, Alpha as long
    GetObject(GetCurrentObject(gP.gldc, 7), sizeof(bm), bm)
    pBits = bm.bmBits
    for P = (bm.bmWidth * bm.bmHeight) to 1 step - 1
        Alpha = Rgb2Gray(RGB(@pBits[2],@pBits[1],@pBits[0])) and &H000000FF???
        if (Alpha = 0) then
            @pBits[3] = 0
        else
            @pBits[3] = 255
        end if
        pBits = pBits + 4
    next
    BitBlt(hMemDC, x, y, bm.bmWidth, bm.bmHeight, gP.gldc, 0, 0, %SRCCOPY)
end sub

sub SetImage(byval hWnd as long)
    local x, y as long
    local rw as RECT
    local bf as BLENDFUNCTION
    local lp, ptSrc as POINTAPI
    local lpSize as SIZEL
    
    local hBmp, hMemDC, graphics, DesktopDC, ImgAttr as dword

    static imgW, imgH, xDir, yDir as long
      
    GetWindowRect(hWnd, rw)
    lpSize.Cx = rw.nRight - rw.nLeft: lpSize.Cy = rw.nBottom - rw.nTop
    lp.X = rw.nLeft: lp.Y = rw.nTop

    DesktopDC = GetDC(0)
    
    '// Create our memory DC
    hMemDC = CreateCompatibleDC(DesktopDC)
    hBmp = CreateDIB(DesktopDC, lpSize.Cx, lpSize.CY, 32)
    if (hBmp) then
        SelectObject(hMemDC, hBmp)
        
        if (gP.img = 0) then
            GdipLoadImageFromFile((ucode$(EXE.Path$ + "Shadow.png")), gP.img)
            if (gP.img) then GetImageSize(gP.img, imgW, imgH)
        end if
        
        '// Draw the OpenGL scene
        DrawTheScene()
        wglMakeCurrent(gP.gldc, gP.glrc)
        SwapBuffers(gP.gldc)
        DrawAndSetupAlphaChannel(hMemDC, -13, -14)
       
        if (gP.img) then
            if (GdipCreateFromHDC(hMemDC, graphics) = 0) then
                GdipSetInterpolationMode(graphics, 2)
                GdipDrawImageRectRectI(graphics, gP.img, 1, 14, imgW, imgH, 0, 0, imgW, imgH, 2, ImgAttr)
                GdipDeleteGraphics(graphics)
            end if
        end if    
       
        '// Update the layered window    
        bf.BlendOp             = %AC_SRC_OVER
        bf.BlendFlags          = 0
        bf.AlphaFormat         = %AC_SRC_ALPHA '// Use source alpha
        bf.SourceConstantAlpha = 255 '//alpha
        UpdateLayeredWindow (hWnd, DesktopDC, lp, lpSize, hMemDC, ptSrc, 0, bf, %ULW_ALPHA)
       
        DeleteObject(hBmp)
        DeleteDC(hMemDC)     
       
        '//Move our window around the screen
        if (xDir = 0) then xDir = %MOVE_STEP: yDir = %MOVE_STEP
        if (xDir > 0) then
            if (gP.x + %FRAME_SizeX - 21 > GetSystemMetrics(%SM_CXSCREEN)) then xDir = -xDir: PlayWav()
        elseif (gP.x < 0) then 
            xDir = -xDir: PlayWav()
        end if
        if (yDir > 0) then
            if (gP.y + %FRAME_SizeY - 21 > GetSystemMetrics(%SM_CYSCREEN)) then yDir = -yDir: PlayWav()
        elseif (gP.y < 0) then 
            yDir = -yDir: PlayWav()
        end if
        gP.x = gP.x + xDir: gP.y = gP.y + yDir
        MoveWindow(hWnd, gP.x, gP.y, %FRAME_SizeX, %FRAME_SizeY, 0)
    
    end if
    ReleaseDC(0, DesktopDC)
end sub

sub DrawTheScene() ' See MSDN documentation for the use of the OpenGL API
    static xrot as single, yrot as single, zrot as single

    glClear(%GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT)

    glPushMatrix()
         glTranslatef(0.0, 0.0, -5.26) 
         glRotatef(xrot, 1.0, 0.0, 0.0)                       ' Rotate On The X Axis
         glRotatef(yrot, 0.0, 1.0, 0.0)                       ' Rotate On The Y Axis
         glRotatef(zrot, 0.0, 0.0, 1.0)                       ' Rotate On The Z Axis

         xrot += 0.6 * rnd(3,6) ' X Axis Rotation
         yrot += 0.4 * rnd(3,6) ' Y Axis Rotation
         zrot += 0.8 * rnd(4.6) ' Z Axis Rotation

         glColor4ub(255,255,255,255)

         glCallList(gP.sphere)
    glPopMatrix()


end sub

function SetupPixelFormat(byval hDC as long) as long
    local pfd as PIXELFORMATDESCRIPTOR
    local pixelformat as long
    local lRet as long

    lRet = %TRUE

    pfd.nSize      = sizeof(pfd)
    pfd.nVersion   = 1
    pfd.dwFlags    = %PFD_SUPPORT_OPENGL OR %PFD_DRAW_TO_BITMAP
    pfd.iPixeltype = %PFD_TYPE_RGBA

    pfd.cColorBits = 32
    pfd.cDepthBits = 32

    pixelformat = ChoosePixelFormat(hDC, pfd)

    if (pixelformat) then
        if SetPixelFormat(hDC, pixelformat, pfd) = 0 then lRet = %FALSE
    else
        lRet = %FALSE
    end if

    function = lRet
end function

function CreateGLcontext(byval nWidth as long, byval nHeight as long) as long
    local nRet, DesktopDC, hGLBMP as long
    if ((nWidth > 0) and (nHeight > 0)) then
        DesktopDC = GetDC(0)
        gP.gldc = CreateCompatibleDC(DesktopDC)
        hGLBMP = CreateDIB(DesktopDC, nWidth, nHeight, 32)
        SelectObject(gP.gldc, hGLBMP)
        ReleaseDC(0, DesktopDC)
        if (hGLBMP) then
            if (SetupPixelFormat(gP.gldc)) then
                gP.glrc = wglCreateContext(gP.gldc)
                if (gP.glrc) then
                    if (wglMakeCurrent(gP.gldc, gP.glrc)) then
                   
                        glClearColor(0.0, 0.0, 0.0, 0.0)
                        glClearDepth(1.0)
                        glDepthFunc(%GL_LESS)'GREATER)'LEQUAL)'ALWAYS)
                        glEnable(%GL_DEPTH_TEST)
                        glShadeModel(%GL_SMOOTH)
                        
                        glMatrixMode(%GL_MODELVIEW)'%GL_PROJECTION)
                        
                        glViewport(0, 0, nWidth, nHeight)
                        
                        glEnable(%GL_TEXTURE_2D)
                        
                        '// Use alphablending
                        glEnable(%GL_BLEND)
                        glBlendFunc(%GL_SRC_ALPHA, %GL_ONE_MINUS_SRC_ALPHA)
                        
                        
                        dim LightAmbient(3) as single:  array assign LightAmbient()  = 0.05, 0.05, 0.05, 1.0
                        dim LightDiffuse(3) as single:  array assign LightDiffuse()  = 0.6, 0.6, 0.6, 0.1
                        dim LightPosition(3) as single: array assign LightPosition() = -0.20, 0.20, 0.20, 0.7
                        glLightfv(%GL_LIGHT0, %GL_AMBIENT, LightAmbient(0))      ' Setup The Ambient Light
                        glLightfv(%GL_LIGHT0, %GL_DIFFUSE, LightDiffuse(0))      ' Setup The Diffuse Light
                        glLightfv(%GL_LIGHT0, %GL_POSITION, LightPosition(0))    ' Position The Light
                        glEnable(%GL_LIGHT0)                                     ' Enable Light ZERO
                        glEnable(%GL_LIGHTING)                                   ' Enable Lighting
                        glEnable(%GL_COLOR_MATERIAL)                             ' Enable Coloring Of Material
                        
                        gluPerspective(24, 1.0, 1.0, 100.0)
                        
                        nRet = -1
                      
                    end if
                end if
            end if
        end if
    end if
    function = nRet
end function

sub DestroyGLresources()
    if (gP.glrc) then
        wglMakeCurrent(gP.gldc, 0)
        wglDeleteContext(gP.glrc): gP.glrc = 0
    end if
    if (gP.gldc) then
        DeleteObject(GetCurrentObject(gP.gldc, %OBJ_BITMAP)) ' Delete the memory bitmap
        DeleteDC(gP.gldc): gP.gldc = 0
    end if
end sub

'// Main window procedure
function WndProc(byval hWnd as long, byval msg as dword, byval wParam as long, byval lParam as long) as long

    select case long msg
        
    case %WM_KEYDOWN:
         if (wParam = %VK_ESCAPE) then DestroyWindow(hWnd)
         
    case %WM_TIMER:
         SetImage(hWnd)
         
    case %WM_NCHITTEST:
         function = %HTCAPTION: EXIT function

    case %WM_DESTROY:
         PostQuitMessage(0)
         function = 0: EXIT function
    end select

    function = DefWindowProc(hWnd, msg, wParam, lParam)

end function

function WinMain (byval hInstance as dword, byval hPrevInstance as dword, byval lpCmdLine as asciiz PTR, byval nCmdShow as long) as long

    local msg as tagMsg, wcx as WNDCLASSEX, rc as RECT, zClass as asciiz * 16
    local hMain as dword, hGDIplus, IsInitialized as long

    zClass = "ZAEROGL"
    wcx.cbSize = sizeof(WNDCLASSEX)
    IsInitialized = GetClassInfoEx(hInstance, zClass, wcx)
    if (IsInitialized     = 0) then
        wcx.style         = 0
        wcx.lpfnWndProc   = CODEPTR(WndProc)
        wcx.cbClsExtra    = 0
        wcx.cbWndExtra    = 0
        wcx.hInstance     = hInstance
        wcx.hIcon         = 0
        wcx.hCursor       = 0
        wcx.hbrBackground = 0
        wcx.lpszMenuName  = 0
        wcx.lpszClassName = VARPTR(zClass)
        wcx.hIconSm       = wcx.hIcon
        if (RegisterClassEx(wcx)) then IsInitialized = -1
    end if

    if (IsInitialized) then
        '// Init GDIPLUS
        hGDIplus = GdipStart()

        gP.x = max&((GetSystemMetrics(%SM_CXSCREEN) - %FRAME_SizeX) \ 2, 0)
        gP.y = max&((GetSystemMetrics(%SM_CYSCREEN) - %FRAME_SizeY) \ 2, 0)

        '// Create The Window
        MyTitle$ = EXE.Name$
        hMain = CreateWindowEx(%WS_EX_LAYERED OR %WS_EX_TOPMOST OR %WS_EX_TOOLWINDOW, _
                               zClass, _               ' Class Name
                               (MyTitle$), _           ' Window Title
                               %WS_POPUP, _            ' Defined Window Style
                               gP.x, gP.y, %FRAME_SizeX, %FRAME_SizeY, _ 
                               %NULL, _                ' No Parent Window
                               %NULL, _                ' No Menu
                               hInstance, _            ' Instance
                               byval %NULL)            ' Dont Pass Anything to WM_CREATE
      
        if (hMain) then
      
            '// The OpenGL stuff
            CreateGLcontext(%FRAME_SizeX, %FRAME_SizeY)
            gP.mt.FullName = EXE.Path$ + "boing2.png"
            gP.mt.ID = 1
            gP.mt.Square = 0
            MakeMultipleTexture (byval VARPTR(gP.mt), 1)

            'PlayWav()
          
            gP.sphere = GLsphere(gP.mt.Texture)
          
            ShowWindow(hMain, %SW_SHOW)
            SetTimer(hMain, 1, %TIMER_DELAY, %NULL)
            WHILE GetMessage(msg, %NULL, 0, 0)
                  TranslateMessage(msg)
                  DispatchMessage(msg)
            WEND
            KillTimer(hMain, 1)
            
            '//FreeUp resources
            glDeleteLists(gP.sphere, 1)
            if gP.img then GdipDisposeImage(gP.img): gP.img = 0
            DestroyGLresources()
          
            function = msg.wParam
        end if
      
        '// UNLOAD GDIPLUS
        GdipEnd(hGDIplus)

    end if

end function

